/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * Purpose: Display a source bitmap, read a Prism Data File, and display the defined prism over the source bitmap. * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

//Copy this to the Command-Line:
//-f +w800 +h800

#version 3.7;

global_settings {assumed_gamma 2.2}

#local LINEAR = 0;
#local QUADRATIC = 1;
#local CUBIC = 2;
#local BEZIER = 3;

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * SET THESE VARIABLES                                                                                             * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#local SourceBitmapFileName = "Bitmap_Spot.png"; //Original Bitmap image for reference

#local PrismDataFileName = "Data_Spot.txt"; //Prism Data File to READ

#local ShowGrid = 1; //Boolean

#local ShowPointMarkers = 1; //Boolean

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * SCENE BASICS                                                                                                    * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "colors.inc"

camera {
   orthographic
   location <0, 0, -1>
   right 1 * x
   scale <1.003, 1.003, 1>
   direction <0, 0, 1>
   look_at <0, 0, 0>
} //camera

#local PrismColor = texture {pigment {color DarkOrchid * .7 transmit .4} finish {emission 1}}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * PRISM () MACRO                                                                                                  * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#macro MakePrism (PrismDataFileName, ShowPointMarkers)

   //Input:
   //   [PrismDataFileName] Name of a prism data file
   //   [ShowPointMarkers] (TRUE/FALSE) Useful only for testing, will show the
   //   locations of the coordinates contained in the prism data file

   //Output:
   //   A prism object called "NewPrism"

   //This macro creates an x, y prism centered at x = 0, y = 0.
   //The prism will extend one unit along the x axis and one unit along the y axis.
   //The front of the prism will be at z = -1 and the back will be at z = 0.
   //The coordinates for the points along the prism outline spline(s) come from an
   //external data file.

   //Prism data files consist of the following items:
   //
   //   A version number
   //
   //   Comments (enclosed in quote marks)
   //
   //   An integer indicating the type of spline to use
   //      0: linear_spline
   //      1: quadratic_spline
   //      2: cubic_spline
   //      3: bezier_spline
   //   Note that at the moment, only types 0 (linear_spline) and 2 (cubic_spline) 
   //   are supported.
   //
   //   Two integers indicating the total number of pixels per row and column of
   //   the original bitmap image
   //
   //   Any number of pairs of <x, y> coordinates
   //
   //All items are separated by commas. The coordinates are generated from the
   //original bitmap image, with the origin at the top left, and define sub-
   //prisms. The data for each sub-prism must end with "-1, -1," and the data for
   //the last sub-prism must end with "-1, -1" (no comma at the end). The first
   //sub-prism is the solid outline. Subsequent sub-prisms create holes in the
   //first sub-prism.

   //Note that integer <x, y> coordinates indicate the center of a pixel. For
   //example, the top left pixel will be <-.5, -.5> to <.5, .5> which is from the
   //top left of the pixel to the bottom right of the pixel.

   #local MaxNDataPairs = 1000000;

   #local DataPair = array [MaxNDataPairs] //<x, y> coordinates

   #local NDataPairs = 0;

   #if (file_exists (PrismDataFileName))
      #fopen PrismDataFile PrismDataFileName read //Automatically closed upon attempt to read past EOF
      #read (PrismDataFile, VersionNumber)
      #read (PrismDataFile, FileComments)
      #read (PrismDataFile, SplineType) //* * * NOT YET USED * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      #read (PrismDataFile, OriginalResolution_X, OriginalResolution_Y) //Total number of pixels per row and column of original bitmap image
      #local OriginalResolution_Max = max (OriginalResolution_X, OriginalResolution_Y);
      #while (defined (PrismDataFile))
         #read (PrismDataFile, PrismDataItem_X, PrismDataItem_Y)
         #local DataPair [NDataPairs] = <PrismDataItem_X, PrismDataItem_Y>;
         #local NDataPairs = NDataPairs + 1;
      #end //#while
   #else
      #error concat ("Error: Prism data file \"", PrismDataFileName "\" does not exist.")
   #end //#if

   #debug concat ("\nReading File:  ", PrismDataFileName, "\n")
   #debug concat ("\nFile Comments: ", FileComments, "\n")

   #debug "\nSpline Type:   "
   #switch (SplineType)
      #case (LINEAR)
         #debug "linear_spline\n\n"
      #break
      #case (QUADRATIC)
         #debug "quadratic_spline\n\n"
      #break
      #case (CUBIC)
         #debug "cubic_spline\n\n"
      #break
      #case (BEZIER)
         #debug "bezier_spline\n\n"
      #break
   #end //#switch

   #for (I, 0, NDataPairs - 1)
      #local CurX = DataPair [I].u;
      #local CurY = DataPair [I].v;
      #if (CurX != -1)
         #local DataPair [I] = <CurX + .5, CurY + .5>; //Adjust data entries so that integer <x, y> coordinates indicate the center of a pixel
      #end //#if
   #end //#for

   #local OutlinePoint = array [NDataPairs * 2]

   #switch (SplineType)
      #case (LINEAR)
         #local NOutlinePoints = 0;
         #local LastStartPoint = 0;
         #for (I, 0, NDataPairs - 1)
            #if (DataPair [I].u = -1)
               #local OutlinePoint [NOutlinePoints] = OutlinePoint [LastStartPoint];
               #local NOutlinePoints = NOutlinePoints + 1;
               #local LastStartPoint = NOutlinePoints;
            #else
               #local OutlinePoint [NOutlinePoints] = DataPair [I];
               #local NOutlinePoints = NOutlinePoints + 1;
            #end //#if
         #end //#for
         #declare NewPrism = object {
            prism {
               linear_spline
               linear_sweep
               0, 1, NOutlinePoints
               #for (I, 0, NOutlinePoints - 1)
                  <OutlinePoint [I].u, -OutlinePoint [I].v>
               #end //#for
            } //prism
            texture {PrismColor} //* * * FOR TESTING PURPOSES; DELETE LATER * * *
         } //object
      #break
      #case (CUBIC)
         #local NOutlinePoints = 0;
         #local LastStartPoint = 0;
         #for (I, 0, NDataPairs - 1)
            #if (DataPair [I].u = -1)
               #local NOutlinePoints = NOutlinePoints + 3;
               #local OutlinePoint [NOutlinePoints - 3] = OutlinePoint [LastStartPoint];
               #local OutlinePoint [NOutlinePoints - 2] = OutlinePoint [LastStartPoint + 1];
               #local OutlinePoint [NOutlinePoints - 1] = OutlinePoint [LastStartPoint + 2];
               #local LastStartPoint = NOutlinePoints;
            #else
               #local OutlinePoint [NOutlinePoints] = DataPair [I];
               #local NOutlinePoints = NOutlinePoints + 1;
            #end //#if
         #end //#for
         #declare NewPrism = object {
            prism {
               cubic_spline
               linear_sweep
               0, 1, NOutlinePoints
               #for (I, 0, NOutlinePoints - 1)
                  <OutlinePoint [I].u, -OutlinePoint [I].v>
               #end //#for
               sturm
            } //prism
            texture {PrismColor} //* * * FOR TESTING PURPOSES; DELETE LATER * * *
         } //object
      #break
   #end //#switch

   #if (ShowPointMarkers) //Only necessary for testing purposes
      #switch (SplineType)
         #case (LINEAR)
            #local NExtraPoints = 2;
         #break
         #case (CUBIC)
            #local NExtraPoints = 4;
         #break
      #end //#switch
      #declare NewPrism = object {
         union {
            object {NewPrism}
            #for (I, 0, NOutlinePoints - NExtraPoints)
               cylinder {
                  <0, 0, 0> <0, 1.3, 0>, .07
                  texture {pigment {color Black} finish {emission 1}}
                  translate <OutlinePoint [I].u, 0, -OutlinePoint [I].v>
               } //cylinder
               cylinder {
                  <0, 0, 0> <0, 1.4, 0>, .05
                  texture {pigment {color Yellow} finish {emission 1}}
                  translate <OutlinePoint [I].u, 0, -OutlinePoint [I].v>
               } //cylinder
            #end //#for
         } //union
      } //object
   #end //#if

   #declare NewPrism = object {
      NewPrism
      rotate -90 * x //Front is at z = -1, back is at z = 0
      scale <1 / OriginalResolution_Max, 1 / OriginalResolution_Max, 1>
      translate <-.5 * OriginalResolution_X / OriginalResolution_Max, .5 * OriginalResolution_Y / OriginalResolution_Max, 0> //Center
   } //object

#end //#macro MakePrism

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * DISPLAY THE SOURCE IMAGE                                                                                        * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#local SourceBitmapImagePigment = pigment {image_map {png SourceBitmapFileName map_type 0 once}}

#local SourceBitmapImageResolution = max_extent (SourceBitmapImagePigment);

#local SourceBitmapImagePigment = pigment {SourceBitmapImagePigment scale <SourceBitmapImageResolution.x, SourceBitmapImageResolution.y, 1>}

#local SourceImage = object {
   box {<0, 0, -.1> <SourceBitmapImageResolution.x, SourceBitmapImageResolution.y, 0> texture {pigment {SourceBitmapImagePigment} finish {emission 1}}}
} //object

#local XOffsetFactor = 0;
#local YOffsetFactor = 0;

#if (SourceBitmapImageResolution.x / 2 = int (SourceBitmapImageResolution.x / 2))
   #local XOffsetFactor = 1;
#end //#if

#if (SourceBitmapImageResolution.y / 2 = int (SourceBitmapImageResolution.y / 2))
   #local YOffsetFactor = 1;
#end //#if

#local XOffset = -SourceBitmapImageResolution.x / 2 - .5 * XOffsetFactor;
#local YOffset = -SourceBitmapImageResolution.y / 2 + .5 * YOffsetFactor;

#local SourceImage = object {
   #if (SourceBitmapImageResolution.x > SourceBitmapImageResolution.y)
      #local Denominator = SourceBitmapImageResolution.x;
   #else
      #local Denominator = SourceBitmapImageResolution.y;
   #end //#if
   SourceImage
   translate <XOffset, YOffset, 0>
   scale <1/Denominator, 1/Denominator, 1>
} //object

object {SourceImage}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * DISPLAY THE PRISM CREATED FROM THE PRISM DATA FILE                                                              * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

MakePrism (PrismDataFileName, ShowPointMarkers) //Ideally, this data should be computer-generated

object {NewPrism translate <-.5/Denominator * XOffsetFactor, .5/Denominator * YOffsetFactor, 1 - .2>}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * SHOW REFERENCE ITEMS                                                                                            * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#local Background = object {
   #local SourceBitmapImageResolution_Max = max (SourceBitmapImageResolution.x, SourceBitmapImageResolution.y);
   box {
      <-.5, -.5, 0> <.5, .5, .1>
      texture {
         pigment {checker color White * .7 color White * .8}
         scale <1/SourceBitmapImageResolution_Max, 1/SourceBitmapImageResolution_Max, 1>
         translate <-.5, -.5, 0>
         finish {emission 1}
      } //texture
   } //box
} //object

object {Background}

#if (ShowGrid)
   #local Grid = object {
      #local SourceBitmapImageResolution_Max = max (SourceBitmapImageResolution.x, SourceBitmapImageResolution.y);
      #local Width_Outer = .004;
      #local Width_Inner = .0015;
      union {
         #for (I, 0, SourceBitmapImageResolution_Max)
            #local Location = -.5 + I / SourceBitmapImageResolution_Max;
            box {<Location - Width_Outer / 2, -.5, -.3> <Location + Width_Outer / 2, .5, 0> texture {pigment {color Black} finish {emission 1}}}
            box {<Location - Width_Inner / 2, -.5, -.4> <Location + Width_Inner / 2, .5, 0> texture {pigment {color White} finish {emission 1}}}
            box {<-.5, Location - Width_Outer / 2, -.3> <.5, Location + Width_Outer / 2, 0> texture {pigment {color Black} finish {emission 1}}}
            box {<-.5, Location - Width_Inner / 2, -.4> <.5, Location + Width_Inner / 2, 0> texture {pigment {color White} finish {emission 1}}}
         #end //#for
      } //union
   } //object
   object {Grid}
#end //#if
