/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * Purpose: Display a source bitmap and generate a Prism Data File.                                                * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

//Copy this to the Command-Line:
//-f +w800 +h800

#version 3.7;

global_settings {assumed_gamma 2.2}

#local LINEAR = 0;
#local QUADRATIC = 1;
#local CUBIC = 2;
#local BEZIER = 3;

#local Version = 1.0;

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * SET THESE VARIABLES                                                                                             * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#local SourceBitmapFileName = "Bitmap_Spot.png"; //Bitmap image to READ

#local PrismDataFileName = "PrismDataFile.txt"; //Prism Data File to WRITE

#local FileComments = "Test Data";

#local SplineType = CUBIC; //Choices are LINEAR, QUADRATIC, CUBIC, BEZIER (Only LINEAR and CUBIC are currently supported)

#local ColorToLookFor = color <1, 0, 0>; //Ignore all other colors. NOTE: USE ONLY ONES AND ZEROS

#local ShowGrid = 1; //Boolean

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * SCENE BASICS                                                                                                    * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "colors.inc"

camera {
   orthographic
   location <0, 0, -1>
   right 1 * x
   scale <1.003, 1.003, 1>
   direction <0, 0, 1>
   look_at <0, 0, 0>
} //camera

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * EVAL_PIGMENT () MACRO                                                                                           * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#macro eval_pigment (PigmentInQuestion, Location) //From functions.inc

   #local fn = function {pigment {PigmentInQuestion}}

   #local Result = (fn (Location.x, Location.y, Location.z));

   Result

#end //#macro eval_pigment

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * DISPLAY THE SOURCE IMAGE                                                                                        * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#local SourceBitmapImagePigment = pigment {image_map {png SourceBitmapFileName map_type 0 once}}

#local SourceBitmapImageResolution = max_extent (SourceBitmapImagePigment);

#local SourceBitmapImagePigment = pigment {SourceBitmapImagePigment scale <SourceBitmapImageResolution.x, SourceBitmapImageResolution.y, 1>}

#local SourceImage = object {
   box {<0, 0, -.1> <SourceBitmapImageResolution.x, SourceBitmapImageResolution.y, 0> texture {pigment {SourceBitmapImagePigment} finish {emission 1}}}
} //object

#local XOffsetFactor = 0;
#local YOffsetFactor = 0;

#if (SourceBitmapImageResolution.x / 2 = int (SourceBitmapImageResolution.x / 2))
   #local XOffsetFactor = 1;
#end //#if

#if (SourceBitmapImageResolution.y / 2 = int (SourceBitmapImageResolution.y / 2))
   #local YOffsetFactor = 1;
#end //#if

#local XOffset = -SourceBitmapImageResolution.x / 2 - .5 * XOffsetFactor;
#local YOffset = -SourceBitmapImageResolution.y / 2 + .5 * YOffsetFactor;

#local SourceImage = object {
   #if (SourceBitmapImageResolution.x > SourceBitmapImageResolution.y)
      #local Denominator = SourceBitmapImageResolution.x;
   #else
      #local Denominator = SourceBitmapImageResolution.y;
   #end //#if
   SourceImage
   translate <XOffset, YOffset, 0>
   scale <1/Denominator, 1/Denominator, 1>
} //object

object {SourceImage}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * CREATE A 2-D ARRAY OF POINTS                                                                                    * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#local PixelGrid = array [SourceBitmapImageResolution.x] [SourceBitmapImageResolution.y]

#for (XIndex, 0, SourceBitmapImageResolution.x - 1)
   #for (YIndex, 0, SourceBitmapImageResolution.y - 1)
      //Adjust sampling point so that integer <x, y> coordinates indicate the center of a pixel.
      #local CurX = XIndex + .5;
      #local CurY = YIndex + .5;
      #local PixelPigment = eval_pigment (SourceBitmapImagePigment, <CurX, CurY, 0>);
      #if (PixelPigment.red = ColorToLookFor.red & PixelPigment.green = ColorToLookFor.green & PixelPigment.blue = ColorToLookFor.blue)
         #local PixelGrid [XIndex] [(SourceBitmapImageResolution.y - 1) - YIndex] = true;
      #else
         #local PixelGrid [XIndex] [(SourceBitmapImageResolution.y - 1) - YIndex] = false;
      #end //#if
   #end //#for
#end //#for

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * SHOW REFERENCE ITEMS                                                                                            * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#local Background = object {
   #local SourceBitmapImageResolution_Max = max (SourceBitmapImageResolution.x, SourceBitmapImageResolution.y);
   box {
      <-.5, -.5, 0> <.5, .5, .1>
      texture {
         pigment {checker color White * .7 color White * .8}
         scale <1/SourceBitmapImageResolution_Max, 1/SourceBitmapImageResolution_Max, 1>
         translate <-.5, -.5, 0>
         finish {emission 1}
      } //texture
   } //box
} //object

object {Background}

#if (ShowGrid)
   #local Grid = object {
      #local SourceBitmapImageResolution_Max = max (SourceBitmapImageResolution.x, SourceBitmapImageResolution.y);
      #local Width_Outer = .004;
      #local Width_Inner = .0015;
      union {
         #for (I, 0, SourceBitmapImageResolution_Max)
            #local Location = -.5 + I / SourceBitmapImageResolution_Max;
            box {<Location - Width_Outer / 2, -.5, -.3> <Location + Width_Outer / 2, .5, 0> texture {pigment {color Black} finish {emission 1}}}
            box {<Location - Width_Inner / 2, -.5, -.4> <Location + Width_Inner / 2, .5, 0> texture {pigment {color White} finish {emission 1}}}
            box {<-.5, Location - Width_Outer / 2, -.3> <.5, Location + Width_Outer / 2, 0> texture {pigment {color Black} finish {emission 1}}}
            box {<-.5, Location - Width_Inner / 2, -.4> <.5, Location + Width_Inner / 2, 0> texture {pigment {color White} finish {emission 1}}}
         #end //#for
      } //union
   } //object
   object {Grid}
#end //#if

#if (ShowGrid)
   #local GridPixelColor = texture {pigment {color <1 - ColorToLookFor.red, 1 - ColorToLookFor.green, 1 - ColorToLookFor.blue>} finish {emission 1}}
   #local Shape = object {
      union {
         #for (XIndex, 0, SourceBitmapImageResolution.x - 1)
            #for (YIndex, 0, SourceBitmapImageResolution.y - 1)
               #if (PixelGrid [XIndex] [(SourceBitmapImageResolution.y - 1) - YIndex])
                  #local CurX = XIndex + .5;
                  #local CurY = YIndex + .5;
                  disc {<CurX, CurY, -.2>, z, .25}
               #end //#if
            #end //#for
         #end //#for
      } //union
      translate <XOffset, YOffset, 0>
      scale <1/Denominator, 1/Denominator, 1>
      texture {GridPixelColor}
   } //object
   object {Shape}
#end //#if
 
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * GENERATE SPLINE CONTROL POINTS                                                                                  * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#local MaxNOutlinePoints = 1000000;

#local OutlinePoint = array [MaxNOutlinePoints] //<x, y> coordinates

#local NOutlinePoints = 0;



//This is the hard part...

//Remember that integer coordinates represent the center of a pixel and pixels are 1 x 1;
//For example, the top left pixel (centered at <0, 0>) will occupy the space <-.5, -.5>
//to <.5, .5> which is from the top left of the pixel to the bottom right of the pixel.

//Also, the origin is at the top left and positive axis values are to the right and down.
//This is also true in the PixelGrid array, so no conversions are necessary from this
//point on.





/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * WRITE THE CONTROL POINTS TO A PRISM DATA FILE                                                                   * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef (FileComments)
   #local FileComments = "* * * NO COMMENTS * * *";
#end //#if

#if (FileComments = "")
   #local FileComments = "* * * NO COMMENTS * * *";
#end //#if

#debug concat ("\nWriting File:  ", PrismDataFileName, "\n")
#debug concat ("\nFile Comments: ", FileComments, "\n")

#debug "\nSpline Type:   "
#switch (SplineType)
   #case (LINEAR)
      #debug "linear_spline\n\n"
   #break
   #case (QUADRATIC)
      #debug "quadratic_spline\n\n"
   #break
   #case (CUBIC)
      #debug "cubic_spline\n\n"
   #break
   #case (BEZIER)
      #debug "bezier_spline\n\n"
   #break
#end //#switch

#fopen PrismDataFile PrismDataFileName write

#write (PrismDataFile, str (Version, 0, 1), ",\n\n")
#write (PrismDataFile, "\"", FileComments, "\",\n\n")
#write (PrismDataFile, str (SplineType, 0, 0), ",\n\n")
#write (PrismDataFile, SourceBitmapImageResolution.x, ", ", SourceBitmapImageResolution.y, ",\n\n")

#for (I, 0, NOutlinePoints - 1)
   #write (PrismDataFile, OutlinePoint [I].u, ", ", OutlinePoint [I].v ",\n")
#end //#for

#write (PrismDataFile, "-1, -1\n")

#fclose PrismDataFile
